@@ -6,7 +6,6 @@
 // option. This file may not be copied, modified, or distributed
 // except according to those terms.
 #![crate_name="ndarray"]
-#![cfg_attr(has_deprecated, feature(deprecated))]
 #![doc(html_root_url = "http://bluss.github.io/rust-ndarray/master/")]
 
 //! The `ndarray` crate provides an N-dimensional container for general elements
@@ -39,9 +38,7 @@
 //!   + The crate is continuously developing, and breaking changes are expected
 //!     during evolution from version to version. We adhere to semver,
 //!     but alpha releases break at will.
-//!   + We adopt the newest stable rust features we need. In place methods like `iadd`
-//!     *will be deprecated* when Rust supports `+=` and similar in Rust 1.8.
-//!   + We try to introduce more static checking gradually.
+//!   + We adopt the newest stable rust features we need.
 //! - Performance status:
 //!   + Performance of an operation depends on the memory layout of the array
 //!     or array view. Especially if it's a binary operation, which
@@ -58,9 +55,6 @@
 //! The following crate feature flags are available. They are configured in your
 //! `Cargo.toml`.
 //!
-//! - `assign_ops`
-//!   - Requires Rust 1.8, will be default soon.
-//!   - Enables the compound assignment operators
 //! - `rustc-serialize`
 //!   - Optional, compatible with Rust stable
 //!   - Enables serialization support
@@ -68,20 +62,13 @@
 //!   - Optional and experimental, compatible with Rust stable
 //!   - Enable transparent BLAS support for matrix multiplication. Pluggable
 //!     backend via `blas-sys`.
-//! - `rblas`
-//!   - ***Deprecated:*** replaced by separate crate `ndarray-rblas`
-//!   - Enables `rblas` integration
 //!
-#![cfg_attr(all(feature = "assign_ops", not(has_assign)),
-            feature(augmented_assignments, op_assign_traits))]
 
 #[cfg(feature = "serde")]
 extern crate serde;
 #[cfg(feature = "rustc-serialize")]
 extern crate rustc_serialize as serialize;
 
-#[cfg(feature = "rblas")]
-extern crate rblas;
 #[cfg(feature="blas")]
 extern crate blas_sys;
 
@@ -116,22 +103,17 @@ pub use iterators::{
     AxisChunksIter,
     AxisChunksIterMut,
 };
-#[allow(deprecated)]
-pub use iterators::{
-    OuterIter,
-    OuterIterMut,
-};
 
 pub use arraytraits::AsArray;
-pub use linalg::{LinalgScalar, NdFloat};
+pub use linalg_traits::{LinalgScalar, NdFloat};
 pub use stacking::stack;
 
+pub use shape_builder::{ ShapeBuilder };
+
 mod arraytraits;
 #[cfg(feature = "serde")]
 mod arrayserialize;
 mod arrayformat;
-#[cfg(feature = "rblas")]
-pub mod blas;
 mod data_traits;
 
 pub use data_traits::{
@@ -149,24 +131,18 @@ pub use free_functions::*;
 
 mod indexes;
 mod iterators;
-mod linalg;
+mod linalg_traits;
 mod linspace;
 mod numeric_util;
 mod si;
 mod error;
+mod shape_builder;
 mod stacking;
 
 /// Implementation's prelude. Common types used everywhere.
 mod imp_prelude {
+    pub use prelude::*;
     pub use {
-        Axis,
-        ArrayBase,
-        ArrayView,
-        ArrayViewMut,
-        OwnedArray,
-        RcArray,
-        Ix, Ixs,
-        Dimension,
         RemoveAxis,
         Data,
         DataMut,
@@ -180,6 +156,8 @@ mod imp_prelude {
     pub struct Priv<T>(pub T);
 }
 
+pub mod prelude;
+
 /// Array index type
 pub type Ix = usize;
 /// Array index type (signed)
@@ -383,7 +361,6 @@ pub type Ixs = isize;
 /// - `B @ A` which consumes `B`, updates it with the result, and returns it
 /// - `B @ &A` which consumes `B`, updates it with the result, and returns it
 /// - `C @= &A` which performs an arithmetic operation in place
-///   (requires crate feature `"assign_ops"`)
 ///
 /// The trait [`ScalarOperand`](trait.ScalarOperand.html) marks types that can be used in arithmetic
 /// with arrays directly. For a scalar `K` the following combinations of operands
@@ -393,7 +370,6 @@ pub type Ixs = isize;
 /// - `&A @ K` or `K @ &A` which produces a new `OwnedArray`
 /// - `B @ K` or `K @ B` which consumes `B`, updates it with the result and returns it
 /// - `C @= K` which performs an arithmetic operation in place
-///   (requires crate feature `"assign_ops"`)
 ///
 /// ## Broadcasting
 ///
@@ -436,13 +412,6 @@ pub struct ArrayBase<S, D>
 /// can act as both an owner as the data as well as a lightweight view.
 pub type RcArray<A, D> = ArrayBase<Rc<Vec<A>>, D>;
 
-#[cfg_attr(has_deprecated, deprecated(note="`Array` is deprecated! Renamed to `RcArray`."))]
-/// ***Deprecated: Use `RcArray` instead***
-///
-/// Array where the data is reference counted and copy on write, it
-/// can act as both an owner as the data as well as a lightweight view.
-pub type Array<A, D> = ArrayBase<Rc<Vec<A>>, D>;
-
 /// Array where the data is owned uniquely.
 pub type OwnedArray<A, D> = ArrayBase<Vec<A>, D>;
 
@@ -493,32 +462,6 @@ mod impl_methods;
 impl<A, S, D> ArrayBase<S, D>
     where S: Data<Elem=A>, D: Dimension
 {
-    #[cfg(feature = "rblas")]
-    /// Return `true` if the innermost dimension is contiguous (includes
-    /// the special cases of 0 or 1 length in that axis).
-    fn is_inner_contiguous(&self) -> bool {
-        let ndim = self.ndim();
-        if ndim == 0 {
-            return true;
-        }
-        self.shape()[ndim - 1] <= 1 || self.strides()[ndim - 1] == 1
-    }
-
-    #[cfg(feature = "rblas")]
-    /// If the array is not in the standard layout, copy all elements
-    /// into the standard layout so that the array is C-contiguous.
-    fn ensure_standard_layout(&mut self)
-        where S: DataOwned,
-              A: Clone
-    {
-        if !self.is_standard_layout() {
-            let mut v: Vec<A> = self.iter().cloned().collect();
-            self.ptr = v.as_mut_ptr();
-            self.data = DataOwned::new(v);
-            self.strides = self.dim.default_strides();
-        }
-    }
-
     #[inline]
     fn broadcast_unwrap<E>(&self, dim: E) -> ArrayView<A, E>
         where E: Dimension,
@@ -569,9 +512,11 @@ impl<A, S, D> ArrayBase<S, D>
 }
 
 
-mod impl_numeric;
+mod impl_2d;
 
-mod impl_linalg;
+mod numeric;
+
+pub mod linalg;
 
 mod impl_ops_inplace;
 
@@ -755,3 +700,21 @@ enum ElementsRepr<S, C> {
     Slice(S),
     Counted(C),
 }
+
+
+/// A contiguous array shape of n dimensions.
+///
+/// Either c- or f- memory ordered (*c* a.k.a *row major* is the default).
+#[derive(Copy, Clone, Debug)]
+pub struct Shape<D> {
+    dim: D,
+    is_c: bool,
+}
+
+/// An array shape of n dimensions c-order, f-order or custom strides.
+#[derive(Copy, Clone, Debug)]
+pub struct StrideShape<D> {
+    dim: D,
+    strides: D,
+    custom: bool,
+}
